/*
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 *  Copyright (c) 2011 digitalSTROM.org, Zurich, Switzerland
 *  Author: Andreas Brauchli <andreas.brauchli@aizo.com>
 */

/**
 * A function that converts stored timed events from the old to the new version
 */
function convertOldEvents() {
    LOG.logln('Converting events from old timed events app');
    var ndEvents = Property.getNode('events'),
        events = ndEvents.getChildren(),
        i = 0;

    for (i = 0; i < events.length; ++i) {
        var ndOldParams = events[i].getChild('params'),
            oldParams,
            oldRecurrence,
            oldActions;

        if (!ndOldParams) {
            LOG.logln('Warning: no old params found to convert on old event ' + events[i].name);
            continue;
        }

        try {
            // LOG.logln('Debug: old params: ' + ndOldParams.getValue());
            oldParams = JSON.parse(ndOldParams.getValue());
            // LOG.logln('Debug: old recurrence: ' + decodeURIComponent(oldParams.recurrence));
            oldRecurrence = JSON.parse(decodeURIComponent(oldParams.recurrence));
            // LOG.logln('Debug: old actions: ' + decodeURIComponent(oldParams.actions));
            oldActions = JSON.parse(decodeURIComponent(oldParams.actions));
        } catch (e) {
            LOG.logln('Deleting old event '+ events[i].name +': failed to parse');
            deleteOldEvent(events[i]);
            continue;
        }

        //
        // validity checks
        //

        if (!oldRecurrence
            || !oldActions
            || !oldActions.actions
            || !(oldActions.actions.length > 0)
        ) {
            // delete invalid events
            LOG.logln('Deleting old event '+ events[i].name +': missing recurrence or actions');
            deleteOldEvent(events[i]);
            continue;
        }
        oldActions = oldActions.actions;

        if (! oldRecurrence.triggerTime
            || (oldRecurrence.recurrence == 'irregular'
                && (!oldRecurrence.dates
                    || oldRecurrence.dates.length == 0))
            || (oldRecurrence.recurrence == 'regular_wk'
                && (!oldRecurrence.weekdays
                    || oldRecurrence.weekdays.length == 0))
        ) {
            LOG.logln('Deleting old event '+ events[i].name +': missing time, dates or week days');
            deleteOldEvent(events[i]);
            continue;
        }

        //
        // Conversions
        //

        // new params object
        var params = {
            name: oldParams.name
        };

        // disabled event
        if (oldParams.Disabled === true) {
            params.disabled = true;
        }

        // Convert time from UTC to local time
        // (assuming correctly set timezone on the dSS)
        var utc = new Date();
        var time = oldRecurrence.triggerTime.split(':');
        utc.setUTCHours(parseInt(time[0], 10));
        utc.setUTCMinutes(parseInt(time[1], 10));
        utc.setUTCSeconds(parseInt(time[2], 10));
        params.time = // utc.format('H:i:s');
                    pad(utc.getHours())
                    + ':' + pad(utc.getMinutes())
                    + ':' + pad(utc.getSeconds());

        // recurrence period
        if (oldRecurrence.recurrence == 'regular_wk') {
            params.recurring = true;
        }

        if (params.recurring) {
            // weekdays
            params.recurrence = [];
            var j;
            for (j = 0; j < oldRecurrence.weekdays.length; ++j) {
                var idx = getDayIdx(oldRecurrence.weekdays[j]);
                if (idx !== -1) {
                    params.recurrence.push(idx);
                } else {
                    LOG.logln('Skipping invalid weekday: ' + oldRecurrence.weekdays[j]);
                    continue;
                }
            }
            if (params.recurrence.length == 0) {
                LOG.logln('Deleting old event '+ events[i].name +': no valid weekdays');
                deleteOldEvent(events[i]);
                continue;
            }

        } else {
            // reverse the dates from 13-01-2001 to 2001-01-13
            var dates = oldRecurrence.dates;
            var d;
            params.recurrence = [];
            for (d = 0; d < dates.length; ++d) {
                var date = dates[d].split('-');
                if (date.length != 3
                    || parseInt(date[1], 10) > 12) {
                    LOG.logln('Skipping invalid date');
                    continue;
                }
                params.recurrence.push(
                    pad(parseInt(date[2], 10)) + '-'
                    + pad(parseInt(date[1], 10)) + '-'
                    + pad(parseInt(date[0], 10))
                );
            }
            if (params.recurrence.length == 0) {
                LOG.logln('Deleting old event '+ events[i].name +': no valid dates');
                deleteOldEvent(events[i]);
                continue;
            }
        }

        // Convert old actions
        params.actions = [];
        var a;
        for (a = 0; a < oldActions.length; ++a) {
            var p = {};
            if (oldActions[a].Zone !== undefined) {
                p.zone = parseInt(oldActions[a].Zone, 10);
            }
            if (oldActions[a].Group !== undefined) {
                p.group = oldActions[a].Group === '' ?
                    0 : parseInt(oldActions[a].Group, 10);
            }
            if (oldActions[a].Scene !== undefined) {
                p.scene = parseInt(oldActions[a].Scene, 10);
            }
            if (oldActions[a].Output !== undefined) {
                p.outputValue = parseInt(oldActions[a].Output, 10);
            }
            if (oldActions[a].Device) {
                p.device = oldActions[a].Device;
            }
            if (oldActions[a].Blink) {
                p.blink = true;
            }
            params.actions.push(p);
        }

        // store converted event
        deleteOldEvent(events[i]);
        addEvent(params);
        LOG.logln('Converted event ' + params.idname);
    }
} // convertOldEvents

function deleteOldEvent(eventNode) {
    eventNode.getParent().removeChild(eventNode);
}

function convertWeekdayFormat() {
    LOG.logln('Converting weekday format from previous timed events versions');
    var ndEvents = Property.getNode('events'),
        events = ndEvents.getChildren(),
        i = 0;

    for (i = 0; i < events.length; ++i) {
        var ndRecurring = events[i].getChild('recurring'),
            ndRecurrence = !!ndRecurring && !!ndRecurring.getValue() && events[i].getChild('recurrence'),
            recurrences = !!ndRecurrence && ndRecurrence.getChildren(),
            w = 0,
            r = [];

        if (recurrences) {
            for (w = 0; w < recurrences.length; ++w) {
                // change from monday to sunday based weekday index
                var d = (parseInt(recurrences[w].name, 10) + 1) % 7;
                r.push(d);
            }
            events[i].removeChild(ndRecurrence);
            var ndWeekdays = new Property('events/' + events[i].name + '/weekdays');
            ndWeekdays.setValue(r.sort().join(','));
            ndWeekdays.setFlag('ARCHIVE', true);
        }
    }
}
