/*
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 *  Copyright (c) 2011 digitalSTROM.org, Zurich, Switzerland
 *  Author: Andreas Brauchli <andreas.brauchli@aizo.com>
 */

Ext.namespace('DSS.addon.timedevents');

/**
 * An event panel to display timed events
 * @class DSS.addon.timedevents.Panel
 * @extends Ext.grid.Panel
 */
Ext.define('DSS.addon.timedevents.Panel', {

    extend: 'Ext.grid.Panel',
    requires: ['DSS.component.ActionModel'],
    uses: [
        'DSS.addon.timedevents.Editor',
        'DSS.addon.timedevents.Model',
        'DSS.component.ActionType',
        'DSS.component.TargetType',
        'DSS.json.Event',
        'Ext.button.Button',
        'Ext.data.Model',
        'Ext.data.Store',
        'Ext.form.Panel',
        'Ext.grid.Column',
        'Ext.JSON',
        'Ext.selection.RowModel',
        'Ext.window.Window'
    ],

    config: {
        enableColumnHide: false,
        enableColumnMove: false,
        selType: 'rowmodel',
        viewConfig: { forceFit: true }
    },

    /** {Object} The event store */
    store: null,
    /** {Object} The window holding the timedEventsEditor */
    editorWindow: null,
    /** {Object} The timedEventsEditor inside editorWindow */
    timedEventsEditor: null,
    /** {Object} The currently selected event as DSS.addon.timedevents.Model (or null if none) */
    curSelectedEvent: null,
    /** {Object} DSS JSON class */
    property: 'DSS.json.Property',

    constructor: function(config) {
        this.initConfig(config);
        this.property = Ext.create(this.property, config);
        this.callParent(arguments);
    },

    initComponent: function() {
        var me = this;
        me.store = Ext.create('Ext.data.Store', { model: 'DSS.addon.timedevents.Model' });
        me.store.on('load', function(store) {
            me.property.query('events/*(id,recurring)/recurrence(*)', function(res) {
                Ext.Array.each(res.events, function(o) {
                    var m, recs;
                    if (!o.recurring) {
                        m = store.getById(o.id);
                        if (m) {
                            recs = Ext.Object.getValues(o.recurrence[0]);
                            m.set('recurrence', recs.join(','));
                            m.commit();
                        }
                    }
                });
            });
        });

        me.timedEventsEditor = Ext.create('DSS.addon.timedevents.Editor', me.config);

        me.editorWindow = Ext.create('Ext.window.Window', {
            closeAction: 'hide',
            title: me.appName,
            modal: true, // Exclusive input for the window
            height: 500,
            width: 750,
            layout: 'fit',
            items: me.timedEventsEditor
        });

        me.timedEventsEditor.on({
            save: function(values) {
                var pad = function(i) { return i < 10 ? '0' + i : i; },
                    rec = [],
                    weekdays = [],
                    time = [pad(values.recurrence.hours),
                           pad(values.recurrence.minutes),
                           pad(values.recurrence.seconds)].join(':'),
                    tem;

                if (values.recurrence.recurring) {
                    weekdays = values.recurrence.recurrence;
                } else {
                    rec = Ext.Array.map(values.recurrence.recurrence, function(d) {
                        return Ext.Date.format(d, 'Y-m-d');
                    });
                }

                if (values.recurrence.daytime) {
                    time = values.recurrence.daytime + time;
                }

                tem = Ext.create('DSS.addon.timedevents.Model', {
                    name: values.name,
                    time: time,
                    recurring: values.recurrence.recurring,
                    recurrence: rec.join(','),
                    weekdays: weekdays.join(','),
                    actions: values.actions
                });
                me.editorWindow.disable();

                if (me.curSelectedEvent) {
                    me.curSelectedEvent.update(tem);
                } else {
                    tem.publish(function() {
                        // Note that the event is not certain to have been
                        // processed on the dSS.
                        // It might still be in the event queue.
                        me.store.loadData([tem], true);
                    });
                    // reset the current selection
                    var sel = me.getSelectionModel().getSelection();
                    if (sel.length > 0) {
                        me.curSelectedEvent = sel[0];
                    }
                }
                me.editorWindow.hide();
                me.editorWindow.enable();
                me.timedEventsEditor.resetComponent();
            },
            cancel: function() {
                me.editorWindow.hide();
                // reset the currently selected line (was remove to add a new entry)
                if (!me.curSelectedEvent) {
                    var sel = me.getSelectionModel().getSelection();
                    if (sel.length > 0) {
                        me.curSelectedEvent = sel[0];
                    }
                }
                me.timedEventsEditor.resetComponent();
            }
        }); // timedEventsEditor.on()
        me.on({
            selectionchange: function(view, selections, options) {
                var me = this;
                if (selections.length > 0) {
                    me.curSelectedEvent = selections[0];
                    Ext.getCmp('btn-edit').setDisabled(false);
                    Ext.getCmp('btn-delete').setDisabled(false);
                    var btnDisable = Ext.getCmp('btn-disable');
                    btnDisable.setDisabled(false);
                    me.setDisabledButton(me.curSelectedEvent.get('disabled'));
                } else {
                    me.curSelectedEvent = null;
                    Ext.getCmp('btn-edit').setDisabled(true);
                    Ext.getCmp('btn-disable').setDisabled(true);
                    Ext.getCmp('btn-delete').setDisabled(true);
                }
            }
        });
        me.callParent(arguments);
        me.initPage();
    }, // initComponent

    initPage: function() {
        var me = this,
            aptStore = null,
            onAptStoreLoaded = function() {
                DSS.apartment.SceneStore.preloadCustomNameCache(function() {
                    me.store.load();
                });
            };

        // Utterly ugly hack to fully initialize the component
        me.editorWindow.show();
        me.editorWindow.hide();
        // end of utterly ugly hack

        Ext.getCmp('btn-add').handler = function() {
            me.curSelectedEvent = null;
            me.editorWindow.show();
        };
        Ext.getCmp('btn-edit').handler = function() {
            me.timedEventsEditor.resetComponent(me.curSelectedEvent);
            me.editorWindow.show();
        };
        Ext.getCmp('btn-disable').handler = function() {
            var st = !me.curSelectedEvent.get('disabled');
            me.curSelectedEvent.disable(st, function(m) {
                if (me.curSelectedEvent === m) {
                    me.setDisabledButton(st);
                }
            });
        };
        Ext.getCmp('btn-delete').handler = function() {
            me.curSelectedEvent.kill();
        };
        aptStore = DSS.component.ActionModel.getApartmentStore();
        if (aptStore.isLoading()) {
            me.setLoading(true);
            aptStore.on('load', onAptStoreLoaded);
        } else {
            onAptStoreLoaded();
        }
    },

    viewConfig: {
        stripeRows: true
    },

    tbar: {
        items: [{
            text: _('New event'),
            id: 'btn-add',
            iconCls: 'icon-add-action'
            // handler: function() { /* added in initPage() */ }
        }, {
            text: _('Edit event'),
            id: 'btn-edit',
            iconCls: 'icon-edit-action',
            disabled: true
            // handler: function() { /* added in initPage() */ }
        }, {
            text: _('Disable event'),
            id: 'btn-disable',
            iconCls: 'icon-stop-action',
            disabled: true
            // handler: function() { /* added in initPage() */ }
        }, '->', {
            text: _('Delete event'),
            id: 'btn-delete',
            iconCls: 'icon-delete-action',
            disabled: true
            // handler: function() { /* added in initPage() */ }
        }]
    },

    columns: [{
        id: 'col-status',
        header: _('Active'),
        dataIndex: 'disabled',
        renderer: function(disabled) {
            return '<input type="checkbox" disabled="true"'
                    + (!disabled ? ' checked="true">' : '>');
        },
        resizable: false,
        width: 50
    }, {
        id: 'col-name',
        header: _('Name'),
        dataIndex: 'name',
        width: 200
    }, {
        id: 'col-actions',
        header: _('Action'),
        dataIndex: 'id', // alibi index
        maintainFlex: true,
        flex: 5,
        sortable: false,
        renderer: function(val, metadata, record, colIdx, store, view) {
            var actions = record.actions(),
                num = actions.getCount(),
                res = [],
                i;

            for (i = 0; i < num; ++i) {
                res.push(actions.getAt(i).get('text'));
            }
            return res.join('<br>');
        }
    }, {
        id: 'col-time',
        header: _('Time'),
        dataIndex: 'time',
        renderer: function(val) {
            if (isNaN(val.charAt(0))) {
                var tm;
                if (val.indexOf('sunrise') >= 0) {
                    if (val.indexOf('00:00:00') >= 0 || val.length === 7) {
                        return _('At sunrise');
                    }
                    tm = val.substr(8);
                    if (val.charAt(7) === '+') {
                        return tm + '<br>' + _('after sunrise');
                    } else {
                        return tm + '<br>' + _('before sunrise');
                    }
                } else {
                    if (val.indexOf('00:00:00') >= 0 || val.length === 6) {
                        return _('At sunset');
                    }
                    tm = val.substr(7);
                    if (val.charAt(6) === '+') {
                        return tm + '<br>' + _('after sunset');
                    } else {
                        return tm + '<br>' + _('before sunset');
                    }
                }
            } else {
                return val;
            }
        },
        width: 120
    }, {
        id: 'col-recurrence',
        header: _('Recurrence'),
        dataIndex: 'recurrence',
        flex: 2,
        maintainFlex: true,
        sortable: false,
        renderer: function(val, metadata, record, colIdx, store, view) {
            var recurring = record.get('recurring'),
                te = recurring ? record.get('weekdays') : val,
                recs = te.split(','),
                d;

            if (!te) {
                return '';
            }

            if (recurring) {
                var WEEK = [_('MO'),_('TU'),_('WE'),_('TH'),_('FR'),_('SA'),_('SU')],
                    i, j;

                d = [null, null, null, null, null, null, null];
                Ext.Array.each(recs, function(el) {
                    var i = (parseInt(el, 10) + 6) % 7;
                    d[i] = Ext.String.htmlEncode(WEEK[i]);
                });
                for (i = 0; i < 7; ++i) {
                    j = (i + 6) % 7; // use monday based week
                    if (!d[j]) {
                        d[j] = '<span style="'
                            + 'text-decoration:line-through;color:#aaaaaa">'
                            + Ext.String.htmlEncode(WEEK[j])
                            + '</span>';
                    }
                }
                return d.join(', ');

            } else {
                d = [];
                Ext.Array.each(recs, function(el) {
                    var dt = el.split('-'),
                        date,
                        datestr;
                    dt[0] = parseInt(dt[0], 10);
                    dt[1] = parseInt(dt[1], 10) - 1;
                    dt[2] = parseInt(dt[2], 10);
                    date = new Date(dt[0], dt[1], dt[2]);
                    if (date < (new Date())) {
                        // date lays in the past
                        datestr = '<span style="'
                            + 'text-decoration:line-through;color:#aaaaaa">'
                            + date.toLocaleDateString()
                            + '</span>';
                    } else {
                        datestr = date.toLocaleDateString();
                    }
                    d.push(datestr);
                });
                return d.join('<br>');
            }
        }
    }],
    setDisabledButton: function(val) {
        var btnDisable = Ext.getCmp('btn-disable');
        if (val) {
            btnDisable.setText(_('Enable event'));
            btnDisable.setIconCls('icon-start-action');
        } else {
            btnDisable.setText(_('Disable event'));
            btnDisable.setIconCls('icon-stop-action');
        }
    }
});

/**
 * Timed Events Editor Panel
 * The panel will always have at least one action
 * @class DSS.addon.timedevents.Editor
 * @extends Ext.panel.Panel
 */
Ext.define('DSS.addon.timedevents.Editor', {
    extend: 'Ext.panel.Panel',
    mixins: {
        observable: 'Ext.util.Observable'
    },
    uses: [
        'DSS.component.ActionConfigurator',
        'DSS.component.RecurrenceSelection',
        'Ext.data.Model',
        'Ext.data.Store',
        'Ext.form.FieldSet',
        'Ext.form.field.Text',
        'Ext.form.Panel',
        'Ext.layout.container.Border',
        'Ext.layout.container.Fit'
    ],
    config: {
        appName: ''
    },
    /**
     * Gets the activity configurator component
     * that holds the apartment store
     * be careful of what you do with it, esp. filtering
     * @return {Object} The activity configurator or null if not initialized
     */
    getActivityConfigurator: function() {
        return this.activityConfigurator;
    },
    /** The action configuration component for the actions */
    actionConfigurator: null,
    /** The recurrence selection component */
    recurrenceSelection: null,
    /** Textfield holding the event name */
    eventNameBox: null,
    constructor: function(config) {
        this.initConfig(config);
        this.mixins.observable.constructor.call(config);
        this.callParent(arguments);
    },
    initComponent: function() {
        var me = this;

        me.addEvents({
            /**
             * Fired upon cancel button press
             * @event cancel
             */
            cancel: true,
            /**
             * Fired upon save button press
             * @event save
             * @param values {Object} The event parameters of valid events
             */
            save: true
        });

        me.recurrenceSelection = Ext.create('DSS.component.RecurrenceSelection');
        me.actionConfigurator = Ext.create('DSS.component.ActionConfigurator');
        me.eventNameBox = Ext.create('Ext.form.field.Text', {
            name: 'eventname',
            fieldLabel: _('Event name'),
            allowBlank: false,
            width: 350
        });

        Ext.apply(me, {
            layout: 'border',
            items: [
                {
                    region: 'north',
                    bodyPadding: 5,
                    items: [{
                        xtype: 'fieldset',
                        title: _('Timed Event'),
                        items: [
                            me.eventNameBox,
                            me.recurrenceSelection
                        ]
                    }]
                }, {
                    region: 'center',
                    layout: 'fit',
                    items: [me.actionConfigurator]
                }
            ], // items
            buttons: [
                {
                    text: _('Abort'),
                    handler: function() {
                        me.fireEvent('cancel');
                    }
                }, {
                    text: _('Save'),
                    handler: function() {
                        if (me.isValid()) {
                            me.fireEvent('save', me.getValues());
                        }
                    }
                }
            ]
        }); // Ext.apply

        // Initialize
        this.callParent(arguments);
    }, // initComponent
    /**
     * Reset all values to default (unset) or specified values
     * @param values {Object} (optional) DSS.addon.timedevents.Model values to reset to - default if unset
     */
    resetComponent: function(values) {
        var me = this;

        if (values === undefined) {
            me.actionConfigurator.resetComponent();
            me.eventNameBox.setValue('');
            me.eventNameBox.clearInvalid();
            me.recurrenceSelection.resetComponent();
            Ext.getCmp('btn-add-action').setDisabled(true);

        } else {
            me.actionConfigurator.resetComponent(values.actions().data.items);
            me.eventNameBox.setValue(values.get('name'));
            var ts = values.get('time').split(':'),
                daytime,
                rec = [],
                recurring = values.get('recurring');

            if (isNaN(ts[0].charAt(0))) {
                var signIdx = (ts[0].indexOf('+'));
                if (signIdx < 0) {
                    signIdx = ts[0].indexOf('-');
                }
                daytime = ts[0].substr(0, signIdx + 1);
                ts[0] = ts[0].substr(signIdx + 1);
            }

            if (recurring) {
                rec = values.get('weekdays').split(',');
                rec = Ext.Array.map(rec, function(r) { return parseInt(r, 10); });
            } else {
                rec = values.get('recurrence').split(',');
                rec = Ext.Array.map(rec, function(r) {
                    var dt = r.split('-');
                    dt[0] = parseInt(dt[0], 10);
                    dt[1] = parseInt(dt[1], 10) - 1;
                    dt[2] = parseInt(dt[2], 10);
                    return new Date(dt[0], dt[1], dt[2]);
                });
            }
            var recValues = {
                daytime: daytime,
                hours: ts[0],
                minutes: ts[1],
                seconds: ts[2],
                recurring: recurring,
                recurrence: rec
            };
            me.recurrenceSelection.resetComponent(recValues);
        }
    }, // resetComponent

    isValid: function() {
        var me = this;

        // trigger a validate on the weekdays
        me.recurrenceSelection.isValid();
        return (
            me.eventNameBox.isValid()
            && me.recurrenceSelection.isValid()
            && me.actionConfigurator.isValid()
        );
    },

    /**
     * Returns the currently set form values
     * @return {Object} Object of the form:
     *      name: {String} Name given to the event
     *      recurrence: {Object} Object of the form:
     *          daytime: {String} Null if no daytime set, combination of sunrise+ to sunset- for time offsets
     *          recurring: {Boolean} True for recurring (weekly) events,
     *                              false for non recurring fixed date events
     *          recurrence: {Array} Array of int (the weekdays on recurring) / Ext.Date (non recurring) objects
     *          hours: {Number} Hour component of time
     *          minutes: {Number} Minute component of time
     *          seconds: {Number} Seconds component of time
     *      actions: {Array} Array of DSS.component.ActionModel objects
     */
    getValues: function() {
        return {
            name: this.eventNameBox.getValue(),
            recurrence: this.recurrenceSelection.getValues(),
            actions: this.actionConfigurator.getValues()
        };
    }
});

/**
 * Timed Events storage model
 * @class DSS.addon.timedevents.Model
 * @extends Ext.data.Model
 */
Ext.define('DSS.addon.timedevents.Model', {
    extend: 'Ext.data.Model',
    fields: [
        'id',
        { name: 'isTmpId', type: 'boolean', defaultValue: false, persist: false },
        'name',
        { name: 'disabled', type: 'boolean', defaultValue: false },
        'time',
        { name: 'recurring', type: 'boolean', defaultValue: false },
        'recurrence',
        'weekdays'
    ],
    hasMany: [{ name: 'actions', model: 'DSS.component.ActionModel' }],
    proxy: {
        type: 'ajax',
        url: '/json/property/query?query=/scripts/timed-events'
             + '/events/*(id,name,disabled,time,recurring,weekdays)'
             + '/actions/*(blink,scene,outputValue,group,zone,device)',
        pageParam: undefined,
        limitParam: undefined,
        startParam: undefined,
        reader: {
            type: 'json',
            root: 'result.events',
            successProperty: 'ok'
        }
    },
    /** @private */
    getTmpId: function() {
            return 'tmp' + Math.floor(Math.random() * 100000000000);
    },
    constructor: function(data, id, raw) {
        var me = this,
            rawActions = null,
            actions = [];

        if (!raw) {
            if (!id) {
                data.id = me.getTmpId();
                data.isTmpId = true;
            }
            actions = data.actions;
            delete data.actions;
        } else {
            if (!raw.id) {
                // 0.9.7 and older versions did not have the id field.
                data.id = data.name;
            }
            if (raw.actions) {
                rawActions = raw.actions;
                // delete the array to prevent the Ext parser from parsing
                // these into ActionModels
                delete raw.actions;
            }
            try {
                data.name = decodeURIComponent(raw.name);
            } catch (e) { }
        }

        // Init the model and set default values
        me.callParent(arguments);

        me.actions().clearFilter();

        if (raw) {
            // create actions
            Ext.Array.each(rawActions, function(a) {
                actions.push(DSS.component.ActionModel.reconstructFromIds(a));
            });
        }
        this.actions().loadData(actions);
    },
    /**
     * Get the content of this Model as Object that is ready for
     * serialization and compatible to the dSS format
     * @return {Object} This Model's values
     */
    getParams: function() {
        var actions = [],
            recurrence = [],
            recurring = this.get('recurring');

        this.actions().each(function(a) {
            var act = {};

            if (a.get('targetType') === DSS.component.TargetType.ZONE) {
                act.zone = a.zone().first().getId();
                act.group = a.group().first().getId();
            } else if (a.get('targetType') === DSS.component.TargetType.DEVICE) {
                act.device = a.device().first().getId();
            }

            if (a.get('actionType') === DSS.component.ActionType.ACTIVITY) {
                act.scene = a.scene().first().getId();
            } else if (a.get('actionType') === DSS.component.ActionType.OUTPUT) {
                // rescale to 0 - 255
                act.outputValue = Math.ceil(a.get('outputValue') * 2.55);
            } else if (a.get('actionType') === DSS.component.ActionType.BLINK) {
                act.blink = true;
            }
            actions.push(act);
        });

        if (recurring) {
            recurrence = this.get('weekdays').split(',');
            recurrence = Ext.Array.map(recurrence, function(r) {
                return parseInt(r, 10);
            });
        } else {
            recurrence = this.get('recurrence').split(',');
        }

        return {
            id: this.getId(),
            name: this.get('name'),
            disabled: this.get('disabled'),
            actions: actions,
            recurring: recurring,
            recurrence: recurrence,
            time: this.get('time')
        };
    },

    /**
     * Publish this event to the dSS
     * @param handler {Function} Callback issued when sent to the dSS
     */
    publish: function(handler) {
        if (this.get('isTmpId')) {
            this.publishTempId(handler);
        } else {
            this.publishDefault(handler);
        }
    },

    /**
     * @private
     * Publish this event to the dSS
     */
    publishDefault: function(handler) {
        var me = this,
            action = Ext.create('DSS.json.Event', {name: 'timed-events'}),
            params = Ext.clone(me.getParams()),
            hdl = {
                failure: function() {
                    Ext.Msg.alert(
                        _('Error'),
                        _('Couldn\'t create timed event on server')
                    );
                }
            };
        params.name = encodeURIComponent(params.name);

        if (Ext.isFunction(handler)) {
            hdl.success = Ext.bind(handler, this, [me]);
        }
        action.raise({
                action_type: 'add',
                params: Ext.JSON.encode(params)
            },
            hdl
        );
    },

    /**
     * @private
     * Publish this event with temporary id to the dSS and reassign
     * the definitive id with this model
     */
    publishTempId: function(handler) {
        var me = this,
            // We have a temporary id, get the real one generated
            // by the dSS on creation before passing it on to the UI
            subscr = Ext.create('DSS.json.EventSubscription'),
            evt = 'timed-events.create.' + me.getId();

        subscr.subscribe(evt, function() {
            me.publishDefault();
            subscr.get(
                function(res) {
                    var i,
                        id;

                    if (res) {
                        for (i = 0; i < res.length; ++i) {
                            if (res[i].name === evt && res[i].properties) {
                                id = res[i].properties.id;
                                break;
                            }
                        }
                    }
                    if (id) {
                        me.setId(id);
                        me.commit();
                        Ext.callback(handler, me);
                    } else {
                        Ext.Msg.alert(
                            _('Event possibly not created'),
                            _('Please reload this page before continuing')
                        );
                    }
                    subscr.unsubscribeAll();
                },
                10000 // 10s timeout
            );
        });
    },

    /**
     * Update this event on the dSS using a new model
     * Note that the id will be copied over to the new model
     * @param newmodel The new model with the edited values
     */
    update: function(newmodel) {
        var me = this,
            st = me.store,
            id = me.getId(),
            index = me.store.indexOf(me);

        if (isNaN(parseInt(id, 10))) {
            // convert the old id from pre 0.9.8 versions
            // for this we need to delete the old event manually
            // from the dSS but we only do so after publishing the new
            newmodel.commit();
            newmodel.publish(function() {
                me.kill(function() {
                    st.insert(index, [newmodel]);
                });
            });

        } else {
            me.store.remove(me);
            newmodel.setId(id);
            newmodel.set('isTmpId', false);
            newmodel.commit();
            newmodel.publish(function() {
                st.insert(index, [newmodel]);
            });
        }
    },
    /**
     * Delete this event on the dSS
     */
    kill: function(callback) {
        var me =this,
            event = Ext.create('DSS.json.Event', {name: 'timed-events'});

        event.raise(
            {
                action_type: 'delete',
                event_id: me.getId()
            },
            function() {
                me.store.remove(me);
                if (Ext.isFunction(callback)) {
                    Ext.callback(callback);
                }
            }
        );
    },
    /**
     * Enable or disable this event on the dSS
     * @param val {Boolean} True to enable, false to disable
     * @param callback {Function} Callback with this model as argument issued when sent to the dSS
     */
    disable: function(val, callback) {
        var me = this;

        if (me.get('disabled') == val) {
            if (Ext.isFunction(callback)) {
                Ext.callback(callback, me, [me]);
            }
            return;
        }
        var event = Ext.create('DSS.json.Event', {name: 'timed-events'});
        event.raise({
                action_type: (val ? 'disable' : 'enable'),
                event_id: me.getId()
            },
            function() {
                me.set('disabled', val);
                me.commit();
                if (Ext.isFunction(callback)) {
                    Ext.callback(callback, me, [me]);
                }
            }
        );
    }
});
